/*
 *		Manager and monitor for buggy arlan655. :-)
 *
 *	Copyright (c) 2002 by Andy Pershin <apa@puug.org.ru>
 *
 *	This program is free software; you can redistribute it
 *	and/or  modify it under  the terms of  the GNU General
 *	Public  License as  published  by  the  Free  Software
 *	Foundation;  either  version 2 of the License, or  (at
 *	your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 * $Id: arland.c,v 1.1 2002/09/23 06:46:42 apa Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include "gwbox.h"

#define PROC_ARLAN "/proc/arlan"
#define PROC_SYSTEMID "systemId"                         
#define PROC_REGISTRATIONMODE "registrationMode"         
#define PROC_CHANNELNUMBER "channelNumber"               
#define PROC_SPEED "spreadingCode"
#define PROC_RESET "reset0"

static char *argv0;

static void quit(const int sig) {

    log(argv0, LOG_NOTICE, "Killed by signal %d", sig);
    exit(0);
}

static int updproc(char *data, const char *dev, const char *target) {

    char buf[128];
    FILE *fp;
    char *mode = "w";

    snprintf(buf, sizeof(buf)-1, "%s/%s/%s", PROC_ARLAN, dev, target);
    if (data == NULL) mode = "r";
    if ((fp = fopen(buf, mode)) == NULL) {
        log(argv0, LOG_ERR, "buf: %s", strerror(errno));
        return -1;
    }
    if (data != NULL) {
        log(argv0, LOG_INFO, "%s -> %s", data, target);
        fputs(data, fp);
        fflush(fp);
    } else {
        log(argv0, LOG_INFO, "Resetting %s", dev);
        fgetc(fp);
    }
    fclose(fp);

    return 0;
}

static int setup_arlan(const char *dev, const char *params) {

    char *p = strdup(params);
    char *sid, *mode, *channel, *speed;

#define suxx() log(argv0, LOG_ERR, "Wrong params: %s", params)

    if (p == NULL) {
        log(argv0, LOG_ERR, "strdup(): %s", strerror(errno));
        return -1;
    }
    sid = p;
    if ((mode = strchr(sid, ':')) == NULL) {
        suxx();
        free(p);
        return -1;
    }
    *mode++ = '\0';
    if ((channel = strchr(mode, ':')) == NULL) {
        suxx();
        free(p);
        return -1;
    }
    *channel++ = '\0';
    if ((speed = strchr(channel, ':')) == NULL) {
        suxx();
        free(p);
        return -1;
    }
    log(argv0, LOG_INFO, "Setting UP %s device", dev);
    if (updproc(sid, dev, PROC_SYSTEMID) < 0 ||
            updproc(mode, dev, PROC_REGISTRATIONMODE) < 0 ||
            updproc(channel, dev, PROC_CHANNELNUMBER) < 0 ||
            updproc(speed, dev, PROC_SPEED) < 0 ||
            updproc(NULL, dev, PROC_RESET) < 0) {
        free(p);
        return -1;
    }
    free(p);
    return 0;
}

int arland_main (int argc, char *argv[]) {

    int i;
    int nofork = 0;
    char *dev = "arlan0";
    int waiting = 60;
    char *pingprog = NULL;

    argv0 = argv[0];

    while ((i = getopt(argc, argv, "p:w:fd:h")) != EOF) {
        switch (i) {
            case 'p':
                pingprog = optarg;
                break;
            case 'w':
                if ((waiting = atoi(optarg)) == 0) {
                    log(argv[0], LOG_NOTICE, "Bad parameter: %s", optarg);
                    waiting = 60;
                }
                break;
            case 'f':
                nofork = 1;
                break;
            case 'd':
                dev = optarg;
                break;
            case 'h':
            default:
                usage(argv[0]);
                return 1;
        }
    }
    if (pingprog == NULL) {
        log(argv[0], LOG_ERR, "-p undefined");
        usage(argv[0]);
        return 1;
    }
    if (optind >= argc) {
        usage(argv[0]);
        return 1;
    }
    if (setup_arlan(dev, argv[optind]) < 0) return 3;

    if (nofork) {
        if (make_daemon(argv[0], &quit) < 0) return 2;
    }

    for (;;) {
        for (i = optind; i < argc; i++) {
            sleep(waiting);
            if (system(pingprog) != 0) {
                i--;
                continue;
            }
            if (setup_arlan(dev, argv[i]) < 0) return 3;
            if (system(pingprog) != 0) {
                i--;
                continue;
            }
        }
    }

    return 0;
}
