/*
 *  NETMASK, NETWORK  BROADCAST  IP   CIDR.
 *
 * :
 * Linux: gcc -Wall -O2 calcip.c -o calcip
 * Solaris: gcc -Wall -O2 calcip.c -o calcip -lsocket -lnsl
 *
 *   :
 * eval `calcip -p ip 10.9.8.7/24`  
 * ipADDRESS="10.9.8.7"
 * ipNETMASK="255.255.254.0"
 * ipNETWORK="10.9.8.0"
 * ipBROADCAST="10.9.9.255"
 *
 * $Log: calcip.c,v $
 * Revision 1.3  2002/09/09 18:46:04  apa
 * Cleanups.
 *
 * Revision 1.2  2002/09/09 18:19:06  apa
 * Copyrights added. :-)
 *
 * Revision 1.1  2002/09/09 02:55:05  apa
 * Placed to CVS.
 *
 *	Copyright (c) 2002 by Andy Pershin <apa@puug.org.ru>
 *
 *	This program is free software; you can redistribute it
 *	and/or  modify it under  the terms of  the GNU General
 *	Public  License as  published  by  the  Free  Software
 *	Foundation;  either  version 2 of the License, or  (at
 *	your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 * $Id: calcip.c,v 1.3 2002/09/09 18:46:04 apa Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "gwbox.h"

int calcip_main(int argc, char *argv[]) {

    unsigned long int addr, mask, network, broadcast;
    char *p = NULL;
    int m;
    char *prefix = NULL;

    while((m = getopt(argc, argv, "p:h")) != EOF) {
        switch(m) {
	    case 'p':
		prefix = optarg;
		break;
	    case 'h':
	    default:
		usage(argv[0]);
		return 1;
	}
    }

    if(optind >= argc) {
	usage(argv[0]);
	return 1;
    }

    p = strchr(argv[optind], '/');
    if(p) *p++ = '\0';
    if((addr = inet_addr(argv[optind])) == INADDR_NONE) {
	fprintf(stderr, "BAD address: %s\n", argv[optind]);
	return 2;
    }
    if(p) {
	if((m = atoi(p)) > 32 || (m == 0 && *p != '0')) {
	    fprintf(stderr, "Bad mask: /%s\n", p);
	    return 3;
	}
	mask = htonl(~((1 << (32 - m)) - 1));
    } else {
	m = ntohl(addr);
	if(IN_CLASSA(m)) {
	    mask = htonl(IN_CLASSA_NET);
	} else if(IN_CLASSB(m)) {
	    mask = htonl(IN_CLASSB_NET);
	} else {
	    mask = htonl(IN_CLASSC_NET);
	}
    }
    fprintf(stdout, "%sADDRESS=\"%s\"\n", (prefix)?prefix:"",
	inet_ntoa(*((struct in_addr *) &addr)));
    fprintf(stdout, "%sNETMASK=\"%s\"\n", (prefix)?prefix:"",
	inet_ntoa(*((struct in_addr *) &mask)));
    network = addr & mask;
    fprintf(stdout, "%sNETWORK=\"%s\"\n", (prefix)?prefix:"",
	inet_ntoa(*((struct in_addr *) &network)));
    broadcast = network | ~mask;
    fprintf(stdout, "%sBROADCAST=\"%s\"\n", (prefix)?prefix:"",
	inet_ntoa(*((struct in_addr *) &broadcast)));
    return 0;
}
