/*
 *		Micro and stupid `cu' clone.
 *
 *	Copyright (c) 2002 by Andy Pershin <apa@puug.org.ru>
 *
 *	This program is free software; you can redistribute it
 *	and/or  modify it under  the terms of  the GNU General
 *	Public  License as  published  by  the  Free  Software
 *	Foundation;  either  version 2 of the License, or  (at
 *	your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 * $Id: cu.c,v 1.2 2002/09/12 19:14:28 apa Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
//#include <sys/select.h>
#include <sys/time.h>

#ifdef __CYGWIN__
#include <sys/select.h>
#include <getopt.h>
#endif

#include "gwbox.h"

static struct termios optio, ostio;

static int fd;

static void sighandler(const int sig) {

    exit(0);
}

/*    ... */
static int setup(char *device, speed_t speed) {

    struct termios stio, ptio;
    struct sigaction sa;

    if ((fd = open(device, O_RDWR | O_NOCTTY | O_NONBLOCK)) < 0) {
        perror(device);
        return -1;
    }

    memset(&sa, 0, sizeof(sa));
    sa.sa_flags |= SA_NOCLDSTOP;
    sa.sa_flags |= SA_RESTART;
    sa.sa_handler = sighandler;
    sigaction(SIGHUP, &sa, NULL);
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);

    /*
     *     
     *     raw
     */
    tcgetattr(STDIN_FILENO, &ostio);
    memcpy(&stio, &ostio, sizeof(stio));
    stio.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
    stio.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
    stio.c_cflag &= ~(CSIZE | PARENB);
    stio.c_cflag |= CS8;
    stio.c_oflag &= ~(OPOST);
    stio.c_cc[VMIN] = 1;              /*     */
    stio.c_cc[VTIME] = 0;
    tcflush(STDIN_FILENO, TCIFLUSH);
    tcsetattr(STDIN_FILENO, TCSANOW, &stio);

    /*
     *   
     */
    tcgetattr(fd, &optio);
    memcpy(&ptio, &optio, sizeof(ptio));
    ptio.c_cflag = speed | CRTSCTS | CS8 | CLOCAL | CREAD;
    ptio.c_iflag = IGNPAR;
    ptio.c_oflag = 0;
    ptio.c_lflag = 0;
    ptio.c_cc[VMIN] = 1;
    ptio.c_cc[VTIME] = 0;
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &ptio);
    return 0;
}

/*      */
static void reset(void) {

    tcflush(STDIN_FILENO, TCIFLUSH);
    tcsetattr(STDIN_FILENO, TCSANOW, &optio);
    tcflush(fd, TCIFLUSH);
    tcsetattr(fd, TCSANOW, &ostio);
    close(fd);
}

static int cu(void) {

    fd_set fds;
    struct timeval tv;
    int ret;
    char buf[BUFSIZ];

    FD_ZERO(&fds);

    for (;;) {
        FD_SET(STDIN_FILENO, &fds);
        FD_SET(fd, &fds);
        tv.tv_sec = 5;
        tv.tv_usec = 0;
        ret = select(fd+1, &fds, NULL, NULL, &tv);
        if (ret) {
            if (FD_ISSET(STDIN_FILENO, &fds)) {
                ret = read(STDIN_FILENO, buf, sizeof(buf));
                switch (ret) {
                    case -1:
                        perror("read()");
                        return 1;
                    case 0:
                        return 0;
                    default:
                        if (*buf == 0x02 && ret == 1) {
                            /* Control-B pressed. */
                            return 0;
                        }
                        write(fd, buf, ret);
                        FD_CLR(STDIN_FILENO, &fds);
                        break;
                }
            } else if (FD_ISSET(fd, &fds)) {
                ret = read(fd, buf, sizeof(buf));
                switch (ret) {
                    case -1:
                        perror("read()");
                        return 1;
                    case 0:
                        return 0;
                    default:
                        write(STDOUT_FILENO, buf, ret);
                        FD_CLR(fd, &fds);
                        break;
                }
            }
        }
    }
    return 0;
}

int cu_main (int argc, char *argv[]) {

    int opt;
    char *device = "/dev/modem";
    speed_t speed = B9600;

    while ((opt = getopt(argc, argv, "hl:s:c:")) != EOF) {
        switch (opt) {
            case 'l':
                device = optarg;
                break;
            case 's':
                switch (atoi(optarg)) {
                    case 300:
                        speed = B300;
                        break;
                    case 600:
                        speed = B600;
                        break;
                    case 1200:
                        speed = B1200;
                        break;
                    case 2400:
                        speed = B2400;
                        break;
                    case 4800:
                        speed = B4800;
                        break;
                    case 9600:
                        speed = B9600;
                        break;
                    case 19200:
                        speed = B19200;
                        break;
                    case 38400:
                        speed = B38400;
                        break;
                    case 57600:
                        speed = B57600;
                        break;
                    case 115200:
                        speed = B115200;
                        break;
                    default:
                        fprintf(stderr, "Wrong speed: %s, using default.\n", optarg);
                        break;
                }
                break;
            case 'c':
                break;
            case 'h':
            default:
		usage(argv[0]);
                return 3;
        }
    }

    fprintf(stderr, "Press Control-B to exit from %s\n", argv[0]);

    if (setup(device, speed) < 0) {
        return 2;
    }
    atexit(reset);
    return cu();
}
