/*
 * $Id: flctl.c,v 1.2 2002/09/09 02:59:45 apa Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>

#include <linux/flash.h>
#include <linux/major.h>

#include "gwbox.h"

#undef TOTAL_INFO_PRINT

#define FLCNT_FLAG_INFO  (1 << 0)
#define FLCNT_FLAG_ERASE (1 << 1)

#define KB 1024

#ifndef MAJOR
#define MAJOR(dev) (((dev)>>8)&0xff)
#define MINOR(dev) ((dev)&0xff)
#endif

extern int flctl_main(int argc, char **argv)
{
    int flags = 0;
    int i, n_arg = 1;
    int minor, major;
    char *fldev;
    int erase_sector = 0;
    int erase_size_sec = -1;
    flash_info_t info_arg;
    flash_region_info_t region_info_arg;
    flash_partition_t partition_info_arg[FLASH_PART_MAX];
    erase_info_t erase_arg;
    struct stat stat_buf;
    int fd;

    if (argc <= n_arg) {
        usage(argv[0]);
        return 1;
    } else {
        fldev = argv[n_arg++];
    }

    if (argc <= n_arg) {
        flags |= FLCNT_FLAG_INFO;
    } else {
        if (!strcmp(argv[n_arg], "info")) {
            n_arg++;
            flags |= FLCNT_FLAG_INFO;
        } else if (!strcmp(argv[n_arg], "erase")) {
            n_arg++;
            flags |= FLCNT_FLAG_ERASE;
            if (argc > n_arg) {
                erase_sector = strtol(argv[n_arg++],0,0);
                if (argc > n_arg) {
                    erase_size_sec = strtol(argv[n_arg++],0,0);
                }
            }
        } else {
            usage(argv[0]);
            return 1;
        }
    }

    if (argc > n_arg) {
        usage(argv[0]);
        return 1;
    }

    fd = open(fldev, O_RDONLY);
    if (fd < 0) {
	perror(fldev);
        usage(argv[0]);
        return 1;
    }

    stat(fldev, &stat_buf);
    major = MAJOR(stat_buf.st_rdev);
    minor = MINOR(stat_buf.st_rdev) & 7;

    if (major != FLASH_MAJOR) {
        fprintf(stderr, "%s not a flash device\n", fldev);
        return 1;
    }

    if (ioctl(fd, FLASHGETINFO, (char *)&info_arg) < 0) {
	perror("FLASHGETINFO");
        return 1;
    }

    if (ioctl(fd, FLASHGETREGIONINFO, (char *)&region_info_arg) < 0) {
        perror("FLASHGETREGIONINFO");
        return 1;
    }
  
    if (ioctl(fd, FLASHGETPARTINFO, (char *)&partition_info_arg) < 0) {
        perror("FLASHGETPARTINFO");
        return 1;
    }

    if (flags & FLCNT_FLAG_INFO) {
#ifdef TOTAL_INFO_PRINT
        printf("\n"
                "size        = %uK\n"
                "bitwidth    = %u\n"
                "addr_shift  = %u\n"
                "chip_config = %u\n"
                "num_regions = %u\n"
                "num_parts   = %u\n",
                info_arg.size / KB,
                info_arg.bitwidth,
                info_arg.addr_shift,
                info_arg.chip_config,
                info_arg.num_regions,
                info_arg.num_parts);

        printf("\n"
                "offset = 0x%lX\n"
                "sector_size = %uK\n"
                "num_sectors = %u\n",
                region_info_arg.offset,
                region_info_arg.sector_size / KB,
                region_info_arg.num_sectors);

        for (i = 0; i < info_arg.num_parts; i++) {
            printf("\nPARTITION %d\n"
                    "start = 0x%lX\n"
                    "size  = %uK\n",
                    i + 1,
                    partition_info_arg[i].start,
                    partition_info_arg[i].size / KB);
        }
#else
        printf("Flash device 1x%ubit %uK, %u sectors x %uK\n",
                info_arg.bitwidth,
                info_arg.size / KB,
                region_info_arg.num_sectors,
                region_info_arg.sector_size / KB);

        for (i = 0; i < info_arg.num_parts; i++) {
            printf("Partition %d: start = 0x%lX(sector %lu), size = %uK(%d sectors)\n",
                    i + 1,
                    partition_info_arg[i].start,
                    partition_info_arg[i].start / region_info_arg.sector_size,
                    partition_info_arg[i].size / KB,
                    partition_info_arg[i].size / region_info_arg.sector_size);
        }
#endif /* TOTAL_INFO_PRINT */
    }

    if (flags & FLCNT_FLAG_ERASE) {
        erase_arg.offset = erase_sector * region_info_arg.sector_size;
        if (erase_size_sec == -1) {
            erase_arg.size = (minor ? partition_info_arg[minor - 1].size:info_arg.size) - erase_arg.offset;
        } else {
            erase_arg.size = erase_size_sec * region_info_arg.sector_size;
        }
        if ((int)erase_arg.offset <= 0) {
            erase_arg.offset = 0;
        }
        if ((int)erase_arg.size <= 0) {
            erase_arg.size = 0;
        }
        printf("Do you want erase %ld sectors(%d...%d) of %s ?[y|N]: ",
                erase_arg.size / region_info_arg.sector_size,
                erase_sector,
                erase_sector + (int)(erase_arg.size / region_info_arg.sector_size) - 1,
                fldev);
        fflush(stdout);
	i = getchar();
        if (i != 'y' && i != 'Y') {
            printf("Flash NOT erased\n");
            return 0;
        }
        fflush(stdout);
        if (ioctl(fd, FLASHERASE, (char *)&erase_arg) < 0) {
	    perror("FLASHERASE");
            return 2;
        }
    }
    return 0;
}
