/*
 *                    SNTP (RFC1361) client.
 *
 *	Copyright (c) 2002 by Andy Pershin <apa@puug.org.ru>
 *
 *	This program is free software; you can redistribute it
 *	and/or  modify it under  the terms of  the GNU General
 *	Public  License as  published  by  the  Free  Software
 *	Foundation;  either  version 2 of the License, or  (at
 *	your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY  or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 * $Id: sntp.c,v 1.2 2002/09/09 18:19:06 apa Exp $
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include "gwbox.h"

#define PORT 123
#define OFFSET 0x83AA7E80	/* Jan 1970 - Jan 1900 in seconds */

struct sntpPacket {
    unsigned char livnmode, stratum, poll, prec;
    unsigned long delay, disp, refid;
    unsigned long reftime[2], origtime[2], rcvtime[2], txtime[2];
};

static int settime(const time_t t) {

    struct timeval tv;

    if (gettimeofday(&tv, NULL) < 0) {
        perror("gettimeofday()");
        return 4;
    }
    if (t != tv.tv_sec) {
        printf("Set system time: %ld seconds: ", t-tv.tv_sec);
        tv.tv_sec = t;
        if (settimeofday(&tv, NULL) < 0) {
            perror("setimeofday()");
            return 4;
        }
    }
    printf("%s", ctime(&t));
    return 0;
}

static int sntp(const char *server) {

    struct sntpPacket pkt;
    int sock;
    struct sockaddr_in saddr;
    struct hostent *h;
    int len;
    time_t t;

    if ((h = gethostbyname(server)) == NULL) {
        herror(server);
        return 1;
    }

    if ((sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0) {
        perror("socket()");
        return 1;
    }
    memset(&pkt, 0, sizeof(pkt));
    memset(&saddr, 0, sizeof(saddr));
    saddr.sin_family = AF_INET;
    saddr.sin_port = htons(PORT);
    memcpy(&saddr.sin_addr, h->h_addr, sizeof(saddr.sin_addr));
    pkt.livnmode = 0x0B;	/* LI: 0, VN: 1, MODE: 3 (client) */
    if (sendto(sock, &pkt, sizeof(pkt), 0,
                (struct sockaddr *) &saddr, sizeof(saddr)) != sizeof(pkt)) {
        perror("sendto()");
        close(sock);
        return 2;
    }
    len = sizeof(pkt);
    if (recvfrom(sock, &pkt, sizeof(pkt), 0,
                (struct sockaddr *) &saddr, &len) != sizeof(pkt)) {
        perror("recfrom()");
        close(sock);
        return 2;
    }
    close(sock);
    if ((pkt.livnmode & 0xC0) == 0xC0 || !pkt.stratum || !pkt.txtime[0]) {
        fprintf(stderr, "Wrong answer from %s\n", server);
        return 3;
    }

    t = ntohl(pkt.txtime[0]) - OFFSET;
    return settime(t);
}

int sntp_main (int argc, char *argv[]) {

    int i;

    while ((i = getopt(argc, argv, "h")) != EOF) {
        switch (i) {
            case 'h':
            default:
		usage(argv[0]);
                return 1;
        }
    }
    if (argv[optind] == NULL) {
	usage(argv[0]);
        return 1;
    }
    return sntp(argv[optind]);
}
